<?php

class FS_Meta
{
  private $fs_boxes;
  private $fs_groups;
  
  public function __construct()
  {
    $this->fs_boxes = FS_Boxes::instance();
    $this->fs_groups = FS_Groups::instance();
    
    $this->build();
    add_action('save_post', array($this, 'save'), 11, 2);
    //add_action('save_page', array($this, 'save'), 11, 2);
  }
  
  function build()
  {
    foreach ($this->fs_boxes->hays() as $key => $box)
    {
      $post_types = $box['post_types'];
      if (is_array($post_types)) {
        foreach ($post_types as $k => $post_type)
        {
          add_meta_box($box['key'], $box['title'], array($this, 'build_box'), $post_type, 'normal', 'high', 
            array(
              'box' => $box,
              'boxes' => $this->fs_boxes->hays(),
              'groups' => $this->fs_groups->hays()
            ));
        }
      }
    }
  }
  
  function build_box($post, $metabox)
  {
    $boxes = $metabox['args']['boxes'];
    $groups = $metabox['args']['groups'];
    $box = $metabox['args']['box'];
    
    $g = $box['groups'];
    if ((sizeof($groups) > 0) && (is_array($g)) && (sizeof($g) > 0))
    {
      $box_id = 'fs-box-' . $box['key'];
      $box['tag_id'] = $box_id;
      $box_groups = $this->get_groups($box, $groups);
      
      echo "<div class='categorydiv fs-box' id='$box_id'>";
      $this->build_tabs($post, $box, $box_groups);
      foreach ($box_groups as $group)
      {
        $this->build_group($post, $box, $group);
      }
      
      echo '<div class="cut"></div></div>';
    }
    else
    {
      echo "<p>No group included</p>";
      return;
    }        
  }
  
  // get a list of groups that belong to a box
  function get_groups($box)
  {
    $arr = array();
    foreach ($box['groups'] as $group_key)
    {
      $group = $this->fs_groups->get($group_key);
      if (is_array($group))
      {
        $group['tag_id'] = $tab_id = $box['tag_id'] . '-' . $group['key'];
        $arr[] = $group;
      }
    }
    sort_by_order(&$arr);
    return $arr;
  }
  
  function build_tabs($post, $box, $box_groups) 
  {
    echo '<ul class="category-tabs">';
    $i = 0;  
    foreach ($box_groups as $group)
    {      
      $group_id = $group['tag_id'];
      $tab_id = $group['tag_id'] . '-tab';
      $title = $group['title'];
      if ($i++ == 0)
        $class = "tabs group-tab";
      else
        $class = "hide-if-no-js group-tab";
      echo "<li class='" . $class . "' id='$tab_id'><a tabindex='3' href='#$group_id'>$title</a></li>";
    }
    echo '</ul>';
  }
  
  function build_group($post, $box, $group)
  {
    echo "<div class='tabs-panel' id='{$group['tag_id']}'>\n" .
         "<table><tbody>";
    if (is_array($group['fields']))
    {
      foreach ($group['fields'] as $field)
      {
        $this->build_field($post, $box, $field);
      }
    }
    echo "</tbody></table></div>";
  }
  
  function build_field($post, $box, $field)
  {
    $field['tag_id'] = 'tag-' . $box['tag_id'] . '-' . $field['key'];
    $field['tag_name'] = META_FIELD . '[' . $field['key'] . ']';
    $field['meta_value'] = field_get_meta($field['key']);
    echo '<tr class="fs-field">';
    $field_type = FS_FieldTypes::instance()->get($field['type']);
    if (is_object($field_type))
      $field_type->show_field($post, $box, $field);
    else
      echo "<th><label></label></th><td>Field builder missing</td>";
    echo '</tr>';
  }
  
  /* Save */
  function save($postID, $post)
  {
    global $fs_save_flag;
    if($fs_save_flag == 1) return $postID;
    $fs_save_flag = 1;
    if (@constant( 'DOING_AUTOSAVE')) return $postID;
		if (!$_POST) return $postID;
		if (!in_array($_POST['action'], array('editpost', 'post'))) return $postID;

		$post_id = esc_attr($_POST['post_ID']);
		if (!$post_id) $post_id = $new_post_id;
		if (!$post_id) return $postID;
		
		if ( $p = wp_is_post_revision($postID)) $postID = $p;  
    $fields = $this->filter_valid_fields($post_type);
    if (is_array($fields))
    {
      foreach ($fields as $field)
      {
        $this->save_field($postID, $field);
      }
    }
    echo "<p>saving to $postID</p>";
  }
    
  function filter_valid_fields($post_type)
  {
    $boxes = FS_Boxes::instance()->hays();
    $fs_groups = FS_Groups::instance();
    $inputs = $_POST[META_FIELD];
    $fields = array();
    foreach ($boxes as $box)
    {
      if (is_array($box['groups']))
      {
        foreach ($box['groups'] as $group_key)
        {
          $group = $fs_groups->get($group_key);
          if (is_array($group['fields']))
          {
            foreach ($group['fields'] as $field)
            {
              if (isset($inputs[$field['key']]))
              {
                $field['meta_value'] = $inputs[$field['key']];
                $fields[] = $field;
              }
            }
          }
        }
      }
    }
    return $fields;
  }  
  
  function save_field($postID, $field)
  {
    $field_type = FS_FieldTypes::instance()->get($field['type']);
    if (is_object($field_type))
      $field_type->save_field($postID, $field);
  }  
}

?>